# -*- coding: utf-8 -*-
# Generated by Django 1.11.11 on 2018-06-15 10:26
from __future__ import unicode_literals

from django.db import migrations, models

from ..settings import PUSH_NOTIFICATIONS_SETTINGS as SETTINGS


class Migration(migrations.Migration):

    dependencies = [
        ('push_notifications', '0006_webpushdevice'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gcmdevice',
            name='registration_id',
            field=models.TextField(unique=SETTINGS['UNIQUE_REG_ID'], verbose_name='Registration ID'),
        ),
        migrations.AlterField(
            model_name='apnsdevice',
            name='registration_id',
            field=models.CharField(unique=SETTINGS['UNIQUE_REG_ID'], max_length=200, verbose_name='Registration ID'),
        ),
        migrations.AlterField(
            model_name='wnsdevice',
            name='registration_id',
            field=models.TextField(unique=SETTINGS['UNIQUE_REG_ID'], verbose_name='Notification URI'),
        ),
        migrations.AlterField(
            model_name='webpushdevice',
            name='registration_id',
            field=models.TextField(unique=SETTINGS['UNIQUE_REG_ID'], verbose_name='Registration ID'),
        ),
    ]
